/* sierpinski gasket with vertex arrays */

#include "Angel.h"

clock_t time0,time1;
float timer010;  // timer counting 0->1->0
bool bUp;        // flag if counting up or down.
GLuint program;

//----------------------------------------------------------------------------
void UpdateTimer()
{
    time1 = clock();
    float delta = static_cast<float>(static_cast<double>(time1-time0)/static_cast<double>(CLOCKS_PER_SEC));
    delta = delta / 4;
    if (delta > 0.00005f)
    {
        time0 = clock();
        if (bUp)
        {
            timer010 += delta;
            if (timer010>=1.0f) { timer010 = 1.0f; bUp = false;}
        }
        else
        {
            timer010 -= delta;
            if (timer010<=0.0f) { timer010 = 0.0f; bUp = true;}
        }
    }
    glutPostRedisplay();
}
//----------------------------------------------------------------------------

void OnResize(int w, int h)
{
    if(h == 0) h = 1;
	   float ratio = 1.0f * (float)w / (float)h;
    
    glMatrixMode(GL_PROJECTION);
	   glLoadIdentity();
    
	   glViewport(0, 0, w, h);
    
    gluPerspective(45,ratio,1,100);
	   glMatrixMode(GL_MODELVIEW);
	   glLoadIdentity();
	   gluLookAt(0.0f,0.0f,4.0f,
                 0.0,0.0,-1.0,
                 0.0f,1.0f,0.0f);
}

//----------------------------------------------------------------------------

void
init( void )
{
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_NORMALIZE);
    
    // Light model parameters:
    // -------------------------------------------
    
    GLfloat lmKa[] = {0.0, 0.0, 0.0, 0.0 };
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmKa);
    
    glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER, 1.0);
    glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 0.0);
    
    // -------------------------------------------
    // Spotlight Attenuation
    
    GLfloat spot_direction[] = {1.0, -1.0, -1.0 };
    GLint spot_exponent = 30;
    GLint spot_cutoff = 180;
    
    glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, spot_direction);
    glLighti(GL_LIGHT0, GL_SPOT_EXPONENT, spot_exponent);
    glLighti(GL_LIGHT0, GL_SPOT_CUTOFF, spot_cutoff);
    
    GLfloat Kc = 1.0;
    GLfloat Kl = 0.0;
    GLfloat Kq = 0.0;
    
    glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION,Kc);
    glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, Kl);
    glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, Kq);
    
    
    // -------------------------------------------
    // Lighting parameters:
    
    GLfloat light_pos[] = {0.0f, 5.0f, 5.0f, 1.0f};
    GLfloat light_Ka[]  = {1.0f, 0.5f, 0.5f, 1.0f};
    GLfloat light_Kd[]  = {1.0f, 0.1f, 0.1f, 1.0f};
    GLfloat light_Ks[]  = {1.0f, 1.0f, 1.0f, 1.0f};
    
    glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
    glLightfv(GL_LIGHT0, GL_AMBIENT, light_Ka);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light_Kd);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light_Ks);
    
    // -------------------------------------------
    // Material parameters:
    
    GLfloat material_Ka[] = {0.5f, 0.0f, 0.0f, 1.0f};
    GLfloat material_Kd[] = {0.4f, 0.4f, 0.5f, 1.0f};
    GLfloat material_Ks[] = {0.8f, 0.8f, 0.0f, 1.0f};
    GLfloat material_Ke[] = {0.1f, 0.0f, 0.0f, 0.0f};
    GLfloat material_Se = 20.0f;
    
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, material_Ka);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, material_Kd);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, material_Ks);
    glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, material_Ke);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, material_Se);
    
    glClearColor(0.5f, 0.5f, 1.0f, 0.0f);
    glShadeModel(GL_SMOOTH);
    glEnable(GL_DEPTH_TEST);
    
    // Load shaders and use the resulting shader program
    program = InitShader( "vertexshader.glsl", "fragmentshader.glsl" );
    glUseProgram( program );
    
    time0 = clock();
    timer010 = 0.0f;
    bUp = true;
    
}

//----------------------------------------------------------------------------

void
display( void )
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
    //timer010 = 0.09; //for screenshot!
    glPushMatrix();
    glRotatef(timer010*360, 0.5, 1.0f, 0.1f);
    glutSolidTeapot(1.0);
    glPopMatrix();
    glutSwapBuffers();
    
    
}
//----------------------------------------------------------------------------

void
keyboard( unsigned char key, int x, int y )
{
    switch ( key ) {
        case 033:
            exit( EXIT_SUCCESS );
            break;
    }
}

//----------------------------------------------------------------------------

int
main( int argc, char **argv )
{
    glutInit( &argc, argv );
    glutInitDisplayMode( GLUT_RGBA | GLUT_DEPTH );
    glutInitWindowSize( 512, 512 );
    glutCreateWindow( "Phong example" );
    glutReshapeFunc(OnResize);
    glutIdleFunc(UpdateTimer);
    init();
    
    glutDisplayFunc( display );
    glutKeyboardFunc( keyboard );
    
    glutMainLoop();
    return 0;
}
